package myextra.test;

import java.io.CharArrayWriter;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;


/**
 * <p>Title: the XmlExtractor class</p> <p>Description: extraction of information from BioCASE protocol comliant xml document.</p> <p>August, 2005</p> <p>Company: Botanic Garden and Botanical Museum Berlin-Dahlem</p>
 * @author     Jinling Li
 * @version     1.0
 */
public class XmlExtractor extends DefaultHandler {
    private boolean incontents = false; //flag to help us capture the elements inside element content.
    private CharArrayWriter textBuffer; //buffer for putting extracted element data.
    private String ns = ""; //the string of namespace uri.

    //some strings extracted from xml document.
    private String docStr = "";
    private String diagnostics = ""; // all the diagnostics as a stirng
    private Logger logger;

    /**
     * Constructor
     */
    public XmlExtractor(Logger logger, String protocol) {
        super();

        this.logger = logger;

        if (protocol.equalsIgnoreCase("biocase")) {
            //ns = conf.getBioProtocolNsURI();
            ns = "http://www.biocase.org/schemas/protocol/1.3";
        } else if (protocol.equalsIgnoreCase("digir")) {
            //ns = conf.getDiProtocolNsURI();
            ns = "http://digir.net/schema/protocol/2003/1.0";
        }

        textBuffer = new CharArrayWriter();
        logger.log(Level.DEBUG, this.getClass().getName());
    }

    /**
     * Extract a xml file
     * at the beginning of each new element, erase any text and
     * get some start elements information.
     */
    @Override
    public void startElement(String namespaceURI, String localname,
        String qname, Attributes attributes) throws SAXException {
        textBuffer.reset();

        if (namespaceURI.equals(ns)) {
            // get diagnostic attribute value if this attribute exist.
            if (localname.equals("diagnostic")) {
                if (attributes.getValue(0) != null) {
                    diagnostics += ("code=" + attributes.getValue(0) + ";");
                }

                if (attributes.getValue(1) != null) {
                    diagnostics += ("severity=" + attributes.getValue(1) + ";");
                }
            }

            if (localname.equals("content")) {
                incontents = true;
            }
        }

        if (incontents) {
            //get names and atrributes of the elements that inside content element.
            if (!localname.equals("content")) {
                String attr = "";

                for (int i = 0; i < attributes.getLength(); i++) {
                    attr += (" " + attributes.getQName(i) +
                    replaceInvalidChar(attributes.getLocalName(i)) + "=\"" +
                    replaceInvalidChar(attributes.getValue(i)) + "\"");
                }

                docStr += ("<" + localname + attr + ">");
            }
        }
    }

    /**
     * take some action when we reach the end of selected elements.
     */
    @Override
    public void endElement(String namespaceURI, String localname, String qname)
        throws SAXException {
        if (namespaceURI.equals(ns)) {
            //get value of diagnostic and append it to diagnostics string.
            if (localname.equals("diagnostic")) {
                diagnostics += ("message=" + textBuffer.toString());
                diagnostics += "|";
            }

            // set flag for content.
            if (localname.equals("content")) {
                incontents = false;
            }
        }

        if (incontents && (!localname.equals("content"))) {
            docStr += (replaceInvalidChar(textBuffer.toString()) + "</" +
            localname + ">\r\n");
            textBuffer.reset();
        }
    }

    // When the parser encounters plain text (not XML elements), it calls
    // this method, which accumulates them in a buffer.
    @Override
    public void characters(char[] buffer, int start, int length) {
        textBuffer.write(buffer, start, length);
    }

    /**
     * report an exception
     */
    protected void report(SAXException exception) {
        System.out.println(exception.getMessage());
    }

    /**
     * Accessor for docStr.
     * @return docStr.
     */
    public String getDocStr() {
        return docStr;
    }

    protected String getDiagnostics() {
        if (diagnostics != "") {
            int lastIndex = diagnostics.lastIndexOf("|");

            return diagnostics.substring(0, lastIndex);
        } else {
            //return diagnostics;
            return "diag-test";
        }
    }

    /**
     * Replace invalid xml character.
     * @return givenString
     */
    private String replaceInvalidChar(String givenString) {
        givenString = givenString.replaceAll("&", "&amp;");
        givenString = givenString.replaceAll(">", "&gt;");
        givenString = givenString.replaceAll("<", "&lt;");
        givenString = givenString.replaceAll("'", "&apos;");
        givenString = givenString.replaceAll("\"", "&quot;");

        return givenString;
    }
} // end of XmlExtractor
